@@ -6,22 +6,23 @@
 */
 
 /*
-    channel.cpp  -  description
+    channel.cpp  -  The class that controls a channel
     begin:     Wed Jan 23 2002
     copyright: (C) 2002 by Dario Abatianni
     email:     eisfuchs@tigress.com
 
-    $Id: channel.cpp,v 1.27 2002/12/11 16:25:46 eisfuchs Exp $
+    $Id: channel.cpp,v 1.45 2003/02/05 20:16:55 eisfuchs Exp $
 */
 
 #include <qvbox.h>
 #include <qhbox.h>
 #include <qgrid.h>
 #include <qsizepolicy.h>
-// #include <qcombobox.h>
 #include <qheader.h>
 #include <qregexp.h>
 #include <qtooltip.h>
+#include <qtextcodec.h>
+#include <qsplitter.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -38,25 +39,36 @@ Channel::Channel(QWidget* parent) : ChatWindow(parent)
   nickChangeDialog=0;
   topic="";
 
+  quickButtonsChanged=false;
+  quickButtonsState=false;
+
+  splitterChanged=true;
+  modeButtonsChanged=false;
+  modeButtonsState=false;
+
   setType(ChatWindow::Channel);
 
   // Build some size policies for the widgets
   QSizePolicy hfixed=QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred);
   QSizePolicy hmodest=QSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);
+  QSizePolicy vfixed=QSizePolicy(QSizePolicy::Preferred,QSizePolicy::Fixed);
   QSizePolicy modest=QSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
-//  QSizePolicy onlyHorizontal=QSizePolicy(QSizePolicy::Minimum,QSizePolicy::Preferred);
-//  QSizePolicy onlyVertical=QSizePolicy(QSizePolicy::Preferred,QSizePolicy::MinimumExpanding);
   QSizePolicy greedy=QSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
 
   // (this) The main Box, holding the channel view/topic and the input line
   setSpacing(spacing());
   setMargin(margin());
 
+  splitter=new QSplitter(this);
+  setStretchFactor(splitter,10);
+  splitter->setOpaqueResize(true);
+  
   // The grid for the topic line, Nicks/Ops label, Channel View and Nick list
-  QGrid* topicViewNicksGrid=new QGrid(2,this);
+//  QGrid* topicViewNicksGrid=new QGrid(2,this);
+  QVBox* topicViewNicksGrid=new QVBox(splitter);
   topicViewNicksGrid->setSpacing(spacing());
 
-  /* The box holding the Topic label/line, and the channel modes */
+  // The box holding the Topic label/line, and the channel modes
   QHBox* topicBox=new QHBox(topicViewNicksGrid);
   topicBox->setSpacing(spacing());
 
@@ -66,8 +78,8 @@ Channel::Channel(QWidget* parent) : ChatWindow(parent)
   topicLine->setAutoCompletion(false);
   topicLine->setInsertionPolicy(QComboBox::NoInsertion);
 
-  /* The box holding the channel modes*/
-  QHBox* modeBox=new QHBox(topicBox);
+  // The box holding the channel modes
+  modeBox=new QHBox(topicBox);
   modeBox->setSizePolicy(hfixed);
   modeT=new ModeButton("T",modeBox,0);
   modeN=new ModeButton("N",modeBox,1);
@@ -99,48 +111,47 @@ Channel::Channel(QWidget* parent) : ChatWindow(parent)
 
   limit=new QLineEdit(modeBox);
 
-  nicksOps=new QLabel(i18n("Nicks"),topicViewNicksGrid);
-  nicksOps->setAlignment(AlignVCenter | AlignHCenter);
+  showModeButtons(KonversationApplication::preferences.getShowModeButtons());
 
   setTextView(new IRCView(topicViewNicksGrid,NULL));  // Server will be set later in setServer()
 
+//  nicksOps=new QLabel(i18n("Nicks"),topicViewNicksGrid);
+
   // The box that holds the Nick List and the quick action buttons
-  QVBox* nickListButtons=new QVBox(topicViewNicksGrid);
+  QVBox* nickListButtons=new QVBox(splitter);
+
   nickListButtons->setSpacing(spacing());
 
+//  nicksOps=new QLabel(i18n("Nicks"),topicViewNicksGrid);
+  nicksOps=new QLabel(i18n("Nicks"),nickListButtons);
+  nicksOps->setAlignment(AlignVCenter | AlignHCenter);
+
   nicknameListView=new NickListView(nickListButtons);
   nicknameListView->setSelectionModeExt(KListView::Extended);
   nicknameListView->setAllColumnsShowFocus(true);
-  nicknameListView->setSorting(1,false);
+  nicknameListView->setSorting(1,true);
   nicknameListView->addColumn("",2);
   nicknameListView->addColumn("");
   nicknameListView->addColumn("");
   nicknameListView->header()->hide();
+  nicknameListView->setFont(KonversationApplication::preferences.getListFont());
+
+  // the grid that holds the quick action buttons
+  buttonsGrid=new QGrid(2,nickListButtons);
+  // set hide() or show() on grid
+  showQuickButtons(KonversationApplication::preferences.getShowQuickButtons());
 
-  // The grid that holds the quick action buttons
-  QGrid* buttonsGrid=new QGrid(2,nickListButtons);
   for(int index=0;index<8;index++)
   {
-    // Get the button definition
-    QString buttonText=KonversationApplication::preferences.getButtonList()[index];
-    // Extract button label
-    QString buttonLabel=buttonText.section(',',0,0);
-    // Extract button definition
-    buttonText=buttonText.mid(buttonLabel.length()+1);
-
-    // Build new button
-    QuickButton* newQuickButton=new QuickButton(buttonLabel,buttonText,buttonsGrid);
+    // generate empty buttons first, text will be added by updateQuickButtons() later
+    QuickButton* newQuickButton=new QuickButton("","",buttonsGrid);
     buttonList.append(newQuickButton);
-    // Create tooltip for current button
-    QString toolTip=buttonText.replace(QRegExp("&",false,true),"&amp;").
-                               replace(QRegExp("<",false,true),"&lt;").
-                               replace(QRegExp(">",false,true),"&gt;");
-
-    QToolTip::add(newQuickButton,toolTip);
 
     connect(newQuickButton,SIGNAL (clicked(QString)),this,SLOT (quickButtonClicked(QString)) );
   }
 
+  updateQuickButtons(KonversationApplication::preferences.getButtonList());
+
   // The box holding the Nickname button, Channel input and Log Checkbox
   QHBox* commandLineBox=new QHBox(this);
   commandLineBox->setSpacing(spacing());
@@ -150,10 +161,11 @@ Channel::Channel(QWidget* parent) : ChatWindow(parent)
   logCheckBox=new QCheckBox(i18n("Log"),commandLineBox);
   logCheckBox->setChecked(KonversationApplication::preferences.getLog());
 
-  /* Set the widgets size policies */
+  // Set the widgets size policies
   topicBox->setSizePolicy(greedy);
   topicLabel->setSizePolicy(hfixed);
-  topicLine->setSizePolicy(modest);  // This should prevent the widget from growing too wide
+
+  topicLine->setSizePolicy(vfixed);  // This should prevent the widget from growing too wide
 
   limit->setMaximumSize(40,100);
   limit->setSizePolicy(hfixed);
@@ -181,54 +193,63 @@ Channel::Channel(QWidget* parent) : ChatWindow(parent)
 
   connect(channelInput,SIGNAL (returnPressed()),this,SLOT (channelTextEntered()) );
   connect(channelInput,SIGNAL (nickCompletion()),this,SLOT (completeNick()) );
+  connect(channelInput,SIGNAL (textPasted(QString)),this,SLOT (textPasted(QString)) );
+  
   connect(textView,SIGNAL (newText()),this,SLOT (newTextInView()) );
   connect(textView,SIGNAL (newURL(QString)),this, SLOT (urlCatcher(QString)) );
-  connect(textView,SIGNAL (gotFocus()),channelInput,SLOT (setFocus()) );
+  connect(textView,SIGNAL (gotFocus()),this,SLOT (adjustFocus()) );
+
   connect(nicknameListView,SIGNAL (popupCommand(int)),this,SLOT (popupCommand(int)) );
   connect(nicknameButton,SIGNAL (clicked()),this,SLOT (openNickChangeDialog()) );
+
   connect(topicLine,SIGNAL (topicChanged(const QString&)),this,SLOT (requestNewTopic(const QString&)) );
 
   nicknameList.setAutoDelete(true);     // delete items when they are removed
 
   setLog(KonversationApplication::preferences.getLog());
-// if(mode==singleWindows) show();
 }
 
 Channel::~Channel()
 {
   kdDebug() << "Channel::~Channel()" << endl;
 
-  /* Purge nickname list */
+  KonversationApplication::preferences.setChannelSplitter(splitter->sizes());
+  
+  // Purge nickname list
   Nick* nick=nicknameList.first();
   while(nick)
   {
-    /* Remove the first element of the list */
+    // Remove the first element of the list
     nicknameList.removeRef(nick);
-    /* Again, get the first element in the list */
+    // Again, get the first element in the list
     nick=nicknameList.first();
   }
-  /* Unlink this channel from channel list */
+  // Unlink this channel from channel list
   server->removeChannel(this);
 }
 
-void Channel::setServer(Server* newServer)
-{
-  getTextView()->setServer(newServer);
-  ChatWindow::setServer(newServer);
-}
-
 void Channel::requestNewTopic(const QString& newTopic)
 {
   kdDebug() << "requestNewTopic(" << newTopic << ")" << endl;
 
-  topicLine->setCurrentText(topic);
+  QTextCodec* codec=QTextCodec::codecForName(KonversationApplication::preferences.getCodec());
+  topicLine->setCurrentText(codec->toUnicode(topic));
 
   if(newTopic!=topic) sendChannelText("/TOPIC "+newTopic);
 
   channelInput->setFocus();
 }
 
-/* Will be connected to NickListView::popupCommand(int) */
+void Channel::textPasted(QString text)
+{
+  if(server)
+  {
+    QStringList multiline=QStringList::split('\n',text);
+    for(unsigned int index=0;index<multiline.count();index++) sendChannelText(multiline[index]);
+  }
+}
+
+// Will be connected to NickListView::popupCommand(int)
 void Channel::popupCommand(int id)
 {
   QString pattern;
@@ -274,7 +295,7 @@ void Channel::popupCommand(int id)
     case NickListView::DccSend:
       pattern=KonversationApplication::preferences.getCommandChar()+"DCC SEND %u";
       break;
-  } /* switch */
+  } // switch
 
   if(pattern.length())
   {
@@ -412,7 +433,7 @@ void Channel::setName(const QString& newName)
 
 void Channel::setKey(const QString& newKey)
 {
-  kdDebug() << "Channel Key set to " << newKey << endl;
+//  kdDebug() << "Channel Key set to " << newKey << endl;
   key=newKey;
 }
 
@@ -424,17 +445,21 @@ const QString& Channel::getKey()
 void Channel::channelTextEntered()
 {
   QString line=channelInput->text();
+
   if(line.lower()=="/clear")
-    textView->clear();  // FIXME: to get rid of too wide lines
+    textView->clear();
   else
     if(line.length()) sendChannelText(line);
 
   channelInput->clear();
 }
 
-void Channel::sendChannelText(const QString& line)
+void Channel::sendChannelText(const QString& sendLine)
 {
-  /* Is there something we need to display for ourselves? */
+  QTextCodec* codec=QTextCodec::codecForName(KonversationApplication::preferences.getCodec());
+  QCString line=codec->fromUnicode(sendLine);
+
+  // Is there something we need to display for ourselves?
   QString output=filter.parse(server->getNickname(),line,getName());
   if(output!="")
   {
@@ -444,7 +469,7 @@ void Channel::sendChannelText(const QString& line)
     else if(filter.isQuery()) appendQuery(filter.getType(),output);
     else append(server->getNickname(),output);
   }
-  /* Send anything else to the server */
+  // Send anything else to the server
   server->queue(filter.getServerOutput());
 }
 
@@ -666,7 +691,6 @@ void Channel::updateNicksOps()
 
 void Channel::setTopic(QString& newTopic)
 {
-  /* TODO: Somehow we need the nickname to the corresponding topic displayed */
   appendCommandMessage(i18n("Topic"),i18n("The channel topic is \"%1\".").arg(newTopic));
   if(topic!=newTopic)
   {
@@ -690,7 +714,6 @@ void Channel::setTopic(QString& newTopic)
 
 void Channel::setTopic(QString& nickname,QString& newTopic) // Overloaded
 {
-  /* TODO: Somehow we need the nickname to the corresponding topic displayed */
   if(nickname==server->getNickname())
     appendCommandMessage(i18n("Topic"),i18n("You set the channel topic to \"%1\".").arg(newTopic));
   else
@@ -998,23 +1021,95 @@ void Channel::updateQuickButtons(QStringList newButtonList)
 {
   for(int index=0;index<8;index++)
   {
-    QuickButton* quickButton;
+    // Get the button definition
+    QString buttonText=newButtonList[index];
+    // Extract button label
+    QString buttonLabel=buttonText.section(',',0,0);
+    // Extract button definition
+    buttonText=buttonText.section(',',1);
 
-    QStringList buttonText=QStringList::split(',',newButtonList[index]);
-    quickButton=buttonList.at(index);
-    quickButton->setText(buttonText[0]);
-    quickButton->setDefinition(buttonText[1]);
+    QuickButton* quickButton=buttonList.at(index);
+    quickButton->setText(buttonLabel);
+    quickButton->setDefinition(buttonText);
 
     // Update tool tips
     QToolTip::remove(quickButton);
-    QString toolTip=buttonText[1].replace(QRegExp("&",false,true),"&amp;").
-                                  replace(QRegExp("<",false,true),"&lt;").
-                                  replace(QRegExp(">",false,true),"&gt;");
+    QString toolTip=buttonText.replace(QRegExp("&",false,true),"&amp;").
+                               replace(QRegExp("<",false,true),"&lt;").
+                               replace(QRegExp(">",false,true),"&gt;");
 
     QToolTip::add(quickButton,toolTip);
   }
 }
 
+void Channel::showQuickButtons(bool show)
+{
+  // QT does not redraw the buttons properly when they are not on screen
+  // while getting hidden, so we remember the "soon to be" state here.
+  if(isHidden())
+  {
+    quickButtonsChanged=true;
+    quickButtonsState=show;
+  }
+  else
+  {
+    if(show)
+      buttonsGrid->show();
+    else
+      buttonsGrid->hide();
+  }
+}
+
+void Channel::showModeButtons(bool show)
+{
+  // QT does not redraw the buttons properly when they are not on screen
+  // while getting hidden, so we remember the "soon to be" state here.
+  if(isHidden())
+  {
+    modeButtonsChanged=true;
+    modeButtonsState=show;
+  }
+  else
+  {
+    if(show)
+      modeBox->show();
+    else
+      modeBox->hide();
+  }
+}
+
+void Channel::showEvent(QShowEvent* event)
+{
+  // Suppress Compiler Warning
+  event->type();
+
+  // If the show quick/mode button settings have changed, apply the changes now
+  if(quickButtonsChanged)
+  {
+    quickButtonsChanged=false;
+    showQuickButtons(quickButtonsState);
+  }
+  if(modeButtonsChanged)
+  {
+    modeButtonsChanged=false;
+    showModeButtons(modeButtonsState);
+  }
+  if(splitterChanged)
+  {
+    splitterChanged=false;
+    splitter->setSizes(KonversationApplication::preferences.getChannelSplitter());
+  }
+}
+
+void Channel::updateFonts()
+{
+  kdDebug() << "Channel::updateFonts()" << endl;
+
+  getTextView()->setFont(KonversationApplication::preferences.getTextFont());
+  getTextView()->setPaper(QColor("#"+KonversationApplication::preferences.getTextViewBackground()));
+  nicknameListView->setFont(KonversationApplication::preferences.getListFont());
+}
+
 void Channel::openNickChangeDialog()
 {
   if(!nickChangeDialog)
@@ -1046,3 +1141,10 @@ QList<Nick> Channel::getNickList()
 {
   return nicknameList;
 }
+
+void Channel::adjustFocus()
+{
+  channelInput->setFocus();
+}
+
+#include "channel.moc"
