@@ -11,7 +11,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: dccpanel.h,v 1.7 2002/12/02 08:14:32 eisfuchs Exp $
+  $Id: dccpanel.h,v 1.10 2003/01/18 18:58:59 eisfuchs Exp $
 */
 
 #ifndef DCCPANEL_H
@@ -40,17 +40,19 @@ class DccPanel : public ChatWindow
     DccTransfer* getTransferByPort(QString port,DccTransfer::DccType type);
     DccTransfer* getTransferByName(QString name,DccTransfer::DccType type);
 
+  public slots:
+    void adjustFocus();
+    
   protected slots:
     void acceptDcc();
+    void runDcc();
+    void abortDcc();
     void removeDcc();
     void dccSelected();
 
   protected:
     void setButtons(bool accept,bool abort,bool remove,bool open,bool info);
 
-    int spacing();
-    int margin();
-
     KListView* dccListView;
 
     QPushButton* acceptButton;
