@@ -11,11 +11,12 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: inputfilter.cpp,v 1.22 2002/12/14 13:12:44 eisfuchs Exp $
+  $Id: inputfilter.cpp,v 1.29 2003/02/02 23:41:33 eisfuchs Exp $
 */
 
 #include <qstringlist.h>
 #include <qdatetime.h>
+#include <qregexp.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -29,6 +30,7 @@
 #include "inputfilter.h"
 #include "server.h"
 #include "errorcodes.h"
+#include "konversationapplication.h"
 
 InputFilter::InputFilter()
 {
@@ -49,63 +51,63 @@ void InputFilter::setServer(Server* newServer)
 void InputFilter::parseLine(QString newLine)
 {
   QString trailing="";
-  /* Remove white spaces at the end and beginning */
+  // Remove white spaces at the end and beginning
   newLine=newLine.stripWhiteSpace();
-  /* Find end of middle parameter list */
+  // Find end of middle parameter list
   int pos=newLine.find(" :");
-  /* Was there a trailing parameter? */
+  // Was there a trailing parameter?
   if(pos!=-1)
   {
-    /* Copy trailing parameter */
+    // Copy trailing parameter
     trailing=newLine.mid(pos+2);
-    /* Cut trailing parameter from string */
+    // Cut trailing parameter from string
     newLine=newLine.left(pos);
   }
-  /* Remove all unneccessary white spaces to make parsing easier */
+  // Remove all unneccessary white spaces to make parsing easier
   QString incomingLine=newLine.simplifyWhiteSpace();
 
   QString prefix="";
-  /* Do we have a prefix? */
+  // Do we have a prefix?
   if(incomingLine[0]==':')
   {
-    /* Find end of prefix */
+    // Find end of prefix
     pos=incomingLine.find(' ');
-    /* Copy prefix */
+    // Copy prefix
     prefix=incomingLine.mid(1,pos-1);
-    /* Remove prefix from line */
+    // Remove prefix from line
     incomingLine=incomingLine.mid(pos+1);
   }
 
-  /* Find end of command */
+  // Find end of command
   pos=incomingLine.find(' ');
-  /* Copy command (all lowercase to make parsing easier) */
+  // Copy command (all lowercase to make parsing easier)
   QString command=incomingLine.left(pos).lower();
-  /* Are there parameters left in the string? */
+  // Are there parameters left in the string?
   QStringList parameterList;
   if(pos!=-1)
   {
-    /* Cut out the command */
+    // Cut out the command
     incomingLine=incomingLine.mid(pos+1);
-    /* The rest of the string will be the parameter list */
+    // The rest of the string will be the parameter list
     parameterList=QStringList::split(" ",incomingLine);
   }
-  /* Server command, if no "!" was found in prefix */
+  // Server command, if no "!" was found in prefix
   if(prefix.find('!')==-1 && prefix!=server->getNickname()) parseServerCommand(prefix,command,parameterList,trailing);
   else parseClientCommand(prefix,command,parameterList,trailing);
 }
 
 void InputFilter::parseClientCommand(QString& prefix,QString& command,QStringList& parameterList,QString& trailing)
 {
-  /* Extract nickname fron prefix */
+  // Extract nickname fron prefix
   QString sourceNick=prefix.left(prefix.find("!"));
   QString sourceHostmask=prefix.mid(prefix.find("!")+1);
 
   if(command=="privmsg")
   {
-    /* Channel message */
+    // Channel message
     if(isAChannel(parameterList[0]))
     {
-      /* CTCP message? */
+      // CTCP message?
       if(trailing[0]==1)
       {
         // cut out the CTCP command
@@ -116,17 +118,28 @@ void InputFilter::parseClientCommand(QString& prefix,QString& command,QStringLis
 
         // If it was a ctcp action, build an action string
         if(ctcpCommand=="action")
-          server->appendActionToChannel(parameterList[0],sourceNick,ctcpArgument);
+        {
+          if(!isIgnore(prefix,Ignore::Channel))
+            server->appendActionToChannel(parameterList[0],sourceNick,ctcpArgument);
+        }
         // No known CTCP request, give a general message
-        else server->appendServerMessageToChannel(parameterList[0],"CTCP",i18n("Received unknown CTCP-%1 request from %2 to Channel %3").arg(ctcp).arg(sourceNick).arg(parameterList[0]));
+        else
+        {
+          if(!isIgnore(prefix,Ignore::CTCP))
+            server->appendServerMessageToChannel(parameterList[0],"CTCP",i18n("Received unknown CTCP-%1 request from %2 to Channel %3").arg(ctcp).arg(sourceNick).arg(parameterList[0]));
+        }
       }
       // No CTCP, so it's an ordinary channel message
-      else server->appendToChannel(parameterList[0],sourceNick,trailing);
+      else
+      {
+        if(!isIgnore(prefix,Ignore::Channel))
+          server->appendToChannel(parameterList[0],sourceNick,trailing);
+      }
     }
     // No channel message
     else
     {
-      /* CTCP message? */
+      // CTCP message?
       if(trailing[0]==1)
       {
         // cut out the CTCP command
@@ -138,67 +151,102 @@ void InputFilter::parseClientCommand(QString& prefix,QString& command,QStringLis
         // If it was a ctcp action, build an action string
         if(ctcpCommand=="action")
         {
-          // Check if this nick is already in a query with us
-          Query* query=server->getQueryByName(sourceNick);
-          // If not, create a new one
-          if(!query) server->addQuery(sourceNick,sourceHostmask);
-          // else remember hostmask for this nick, it could have changed
-          else server->addHostmaskToNick(sourceNick,sourceHostmask);
+          // Check if we ignore queries from this nick
+          if(!isIgnore(prefix,Ignore::Channel))
+          {
+            // Check if this nick is already in a query with us
+            Query* query=server->getQueryByName(sourceNick);
+            // If not, create a new one
+            if(!query) server->addQuery(sourceNick,sourceHostmask);
+            // else remember hostmask for this nick, it could have changed
+            else server->addHostmaskToNick(sourceNick,sourceHostmask);
 
-          server->appendActionToQuery(sourceNick,ctcpArgument);
-        }
-        // Maybe it was a version request, so act appropriately
-        else if(ctcpCommand=="version")
-        {
-          server->appendStatusMessage(i18n("CTCP"),i18n("Received Version request from %1.").arg(sourceNick));
-          server->ctcpReply(sourceNick,QString("VERSION Konversation %1 (C)2002 Dario Abatianni and Matthias Gierlings").arg(VERSION));
+            server->appendActionToQuery(sourceNick,ctcpArgument);
+          }
         }
         // DCC request?
         else if(ctcpCommand=="dcc")
         {
-          // Extract DCC type and argument list
-          QString dccType=ctcpArgument.lower().section(' ',0,0);
-          QStringList dccArgument=QStringList::split(' ',ctcpArgument.mid(ctcpArgument.find(" ")+1).lower());
+          if(!isIgnore(prefix,Ignore::DCC))
+          {
+            // Extract DCC type and argument list
+            QString dccType=ctcpArgument.lower().section(' ',0,0);
+            QStringList dccArgument=QStringList::split(' ',ctcpArgument.mid(ctcpArgument.find(" ")+1).lower());
 
-          // Incoming file?
-          if(dccType=="send")
-          {
-            emit addDccGet(sourceNick,dccArgument);
+            // Incoming file?
+            if(dccType=="send")
+            {
+              emit addDccGet(sourceNick,dccArgument);
+            }
+            // Incoming file that shall be resumed?
+            else if(dccType=="accept")
+              {
+              emit resumeDccGetTransfer(sourceNick,dccArgument);
+            }
+            // Remote client wants our sent file resumed
+            else if(dccType=="resume")
+            {
+              emit resumeDccSendTransfer(sourceNick,dccArgument);
+            }
           }
-          // Incoming file that shall be resumed?
-          else if(dccType=="accept")
+        }
+        // Maybe it was a version request, so act appropriately
+        else if(ctcpCommand=="version")
+        {
+          if(!isIgnore(prefix,Ignore::CTCP))
           {
-            emit resumeDccGetTransfer(sourceNick,dccArgument);
-          }
-          // Remote client wants our sent file resumed
-          else if(dccType=="resume")
-          {
-            emit resumeDccSendTransfer(sourceNick,dccArgument);
+            server->appendStatusMessage(i18n("CTCP"),i18n("Received Version request from %1.").arg(sourceNick));
+            server->ctcpReply(sourceNick,QString("VERSION Konversation %1 (C)2002 Dario Abatianni and Matthias Gierlings").arg(VERSION));
           }
         }
         // No known CTCP request, give a general message
-        else server->appendStatusMessage(i18n("CTCP"),i18n("Received unknown CTCP-%1 request from %2").arg(ctcp).arg(sourceNick));
+        else
+        {
+          if(!isIgnore(prefix,Ignore::CTCP))
+            server->appendStatusMessage(i18n("CTCP"),i18n("Received unknown CTCP-%1 request from %2").arg(ctcp).arg(sourceNick));
+        }
       }
-      /* No CTCP, so it's an ordinary query message */
+      // No CTCP, so it's an ordinary query message
       else
       {
-        /* Create a new query (server will check for dupes) */
-        server->addQuery(sourceNick,sourceHostmask);
-        /* else remember hostmask for this nick, it could have changed */
-        server->addHostmaskToNick(sourceNick,sourceHostmask);
-        /* Append this message to the query */
-        server->appendToQuery(sourceNick,trailing);
+        if(!isIgnore(prefix,Ignore::Query))
+        {
+          // Create a new query (server will check for dupes)
+          server->addQuery(sourceNick,sourceHostmask);
+          // else remember hostmask for this nick, it could have changed
+          server->addHostmaskToNick(sourceNick,sourceHostmask);
+          // Append this message to the query
+          server->appendToQuery(sourceNick,trailing);
+        }
       }
     }
   }
   else if(command=="notice")
   {
-    // Channel notice?
-    if(isAChannel(parameterList[0]))
-      server->appendServerMessageToChannel(parameterList[0],i18n("Notice"),i18n("from %1 to %2: %3").arg(sourceNick).arg(parameterList[0]).arg(trailing));
-    // Private notice
-    else
-      server->appendStatusMessage(i18n("Notice"),i18n("from %1 to %2: %3").arg(sourceNick).arg(parameterList[0]).arg(trailing));
+    if(!isIgnore(prefix,Ignore::Notice))
+    {
+      // Channel notice?
+      if(isAChannel(parameterList[0]))
+      {
+        server->appendServerMessageToChannel(parameterList[0],i18n("Notice"),i18n("-%1 to %2- %3").arg(sourceNick).arg(parameterList[0]).arg(trailing));
+      }
+      // Private notice
+      else
+      {
+        // Was this a CTCP reply?
+        if(trailing[0]==1)
+        {
+          // cut 0x01 bytes from trailing string
+          QString ctcp(trailing.mid(1,trailing.length()-2));
+          QString replyReason(ctcp.section(' ',0,0));
+          QString reply(ctcp.section(' ',1));
+          server->appendStatusMessage(i18n("CTCP"),i18n("Received CTCP-%1 reply from %2: %3").arg(replyReason).arg(sourceNick).arg(reply));
+        }
+        // No, so it was a normal notice
+        else
+          server->appendStatusMessage(i18n("Notice"),i18n("-%1- %2").arg(sourceNick).arg(trailing));
+      }
+    }
   }
   else if(command=="join")
   {
@@ -247,10 +295,14 @@ void InputFilter::parseClientCommand(QString& prefix,QString& command,QStringLis
   {
     server->setChannelTopic(sourceNick,parameterList[0],trailing);
   }
-  else if(command=="mode") /* mode #channel -/+ mmm params */
+  else if(command=="mode") // mode #channel -/+ mmm params
   {
     parseModes(sourceNick,parameterList);
   }
+  else if(command=="invite")
+  {
+    server->appendStatusMessage(i18n("Invite"),i18n("%1 invited you into channel %2").arg(sourceNick).arg(trailing));
+  }
   else
   {
     server->appendStatusMessage(command,parameterList.join(" ")+" "+trailing);
@@ -261,7 +313,11 @@ void InputFilter::parseServerCommand(QString& prefix,QString& command,QStringLis
 {
   if(command=="ping")
   {
-    server->queue("PONG "+trailing);
+    QString text;
+    text=(trailing.length()) ? trailing : parameterList.join(" ");
+    text=":"+text;
+    if(prefix.length()) text=prefix+" "+text;
+    server->queue("PONG "+text);
   }
   else if(command=="error :closing link:")
   {
@@ -378,6 +434,10 @@ void InputFilter::parseServerCommand(QString& prefix,QString& command,QStringLis
     when.setTime_t(parameterList[3].toUInt());
     server->appendCommandMessageToChannel(parameterList[1],i18n("Topic"),i18n("Topic was set by %1 on %2.").arg(parameterList[2]).arg(when.toString(Qt::LocalDate)));
   }
+  else if(command==ERR_NOSUCHNICK)
+  {
+    server->appendStatusMessage(i18n("Error"),i18n("%1: No such nick/channel.").arg(parameterList[1]));
+  }
   // Nick already on the server, so try another one
   else if(command==ERR_NICKNAMEINUSE)
   {
@@ -404,6 +464,18 @@ void InputFilter::parseServerCommand(QString& prefix,QString& command,QStringLis
     // Autojoin (for now this must be enough)
     if(server->getAutoJoin()) server->queue(server->getAutoJoinCommand());
   }
+  else if(command==RPL_GLOBALUSERS) // Current global users: 589 Max: 845
+  {
+    QString current(trailing.section(' ',3,3));
+    QString max(trailing.section(' ',5,5));
+    server->appendStatusMessage(i18n("Users"),i18n("Current users on the network: %1 of at most %2.").arg(current).arg(max));
+  }
+  else if(command==RPL_LOCALUSERS) // Current local users: 589 Max: 845
+  {
+    QString current(trailing.section(' ',3,3));
+    QString max(trailing.section(' ',5,5));
+    server->appendStatusMessage(i18n("Users"),i18n("Current users on %1: %2 of at most %3.").arg(prefix).arg(current).arg(max));
+  }
   else if(command==RPL_ISON)
   {
     // Tell server to start the next notify timer round
@@ -418,7 +490,11 @@ void InputFilter::parseServerCommand(QString& prefix,QString& command,QStringLis
   }
   else if(command==RPL_AWAY)
   {
-    server->appendStatusMessage(i18n("Away"),i18n("%1 is away: %2").arg(parameterList[1]).arg(trailing) );
+    server->appendStatusMessage(i18n("Away"),i18n("%1 is away: %2").arg(parameterList[1]).arg(trailing));
+  }
+  else if(command==RPL_INVITING)
+  {
+    server->appendStatusMessage(i18n("Invite"),i18n("You invited %1 into channel %2.").arg(parameterList[1]).arg(parameterList[2]));
   }
   else if(command=="mode")
   {
@@ -426,7 +502,66 @@ void InputFilter::parseServerCommand(QString& prefix,QString& command,QStringLis
   }
   else if(command=="notice")
   {
-    server->appendStatusMessage(i18n("Notice"),i18n("from %1: %2").arg(prefix).arg(trailing));
+    server->appendStatusMessage(i18n("Notice"),i18n("-%1- %2").arg(prefix).arg(trailing));
+  }
+  else if(command==RPL_WHOISUSER)
+  {
+    server->appendStatusMessage(i18n("Whois"),i18n("%1 is %2@%3 (%4)").arg(parameterList[1])
+                                                                      .arg(parameterList[2])
+                                                                      .arg(parameterList[3])
+                                                                      .arg(trailing));
+  }
+  else if(command==RPL_WHOISCHANNELS)
+  {
+    QStringList userChannels;
+    QStringList voiceChannels;
+    QStringList opChannels;
+
+    // get a list of all channels the user is in
+    QStringList channelList=QStringList::split(' ',trailing);
+    channelList.sort();
+
+    // split up the list in channels where they are operator / user / voice
+    for(unsigned int index=0;index<channelList.count();index++)
+    {
+      QString lookChannel=channelList[index];
+      if(lookChannel.startsWith("@")) opChannels.append(lookChannel.mid(1));
+      else if(lookChannel.startsWith("+")) voiceChannels.append(lookChannel.mid(1));
+      else userChannels.append(lookChannel);
+    }
+    if(userChannels.count())
+      server->appendStatusMessage(i18n("Whois"),i18n("%1 is user on channels: %2").arg(parameterList[1]).arg(userChannels.join(" ")) );
+    if(voiceChannels.count())
+      server->appendStatusMessage(i18n("Whois"),i18n("%1 has voice on channels: %2").arg(parameterList[1]).arg(voiceChannels.join(" ")) );
+    if(opChannels.count())
+      server->appendStatusMessage(i18n("Whois"),i18n("%1 is operator on channels: %2").arg(parameterList[1]).arg(opChannels.join(" ")) );
+  }
+  else if(command==RPL_WHOISSERVER)
+  {
+    server->appendStatusMessage(i18n("Whois"),i18n("%1 is online via %2 (%3)").arg(parameterList[1])
+                                                                              .arg(parameterList[2])
+                                                                              .arg(trailing) );
+  }
+  else if(command==RPL_WHOISIDENTIFY)
+  {
+    server->appendStatusMessage(i18n("Whois"),i18n("%1 has identified for this nick.").arg(parameterList[1]));
+  }
+  else if(command==RPL_WHOISIDLE)
+  {
+    server->appendStatusMessage(i18n("Whois"),i18n("%1 is idle since %2 seconds.").arg(parameterList[1])
+                                                                                  .arg(parameterList[2]));
+    if(parameterList.count()==4)
+    {
+      QDateTime when;
+      when.setTime_t(parameterList[3].toUInt());
+
+      server->appendStatusMessage(i18n("Whois"),i18n("%1 is online since %2.").arg(parameterList[1])
+                                                                              .arg(when.toString(Qt::LocalDate)));
+    }
+  }
+  else if(command==RPL_ENDOFWHOIS)
+  {
+    server->appendStatusMessage(i18n("Whois"),i18n("End of WHOIS list."));
   }
   // All yet unknown messages go into the frontmost window unaltered
   else
@@ -478,3 +613,22 @@ bool InputFilter::isAChannel(QString check)
 
   return (initial=='#' || initial=='&' || initial=='+' || initial=='!');
 }
+
+bool InputFilter::isIgnore(QString sender,Ignore::Type type)
+{
+  bool doIgnore=false;
+
+  QPtrList<Ignore> list=KonversationApplication::preferences.getIgnoreList();
+
+  for(unsigned int index=0;index<list.count();index++)
+  {
+    Ignore* item=list.at(index);
+    QRegExp ignoreItem(item->getName(),false,true);
+    if(ignoreItem.exactMatch(sender) && (item->getFlags() & type)) doIgnore=true;
+    if(ignoreItem.exactMatch(sender) && (item->getFlags() & Ignore::Exception)) return false;
+  }
+
+  return doIgnore;
+}
+
+#include "inputfilter.moc"
