@@ -6,12 +6,12 @@
 */
 
 /*
-  ircinput.h  -  description
+  ircinput.h  -  The line input widget with chat enhanced functions
   begin:     Tue Mar 5 2002
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: ircinput.h,v 1.2 2002/07/20 15:18:19 eisfuchs Exp $
+  $Id: ircinput.h,v 1.4 2003/01/17 18:00:51 eisfuchs Exp $
 */
 
 #ifndef IRCINPUT_H
@@ -32,17 +32,21 @@ class IRCInput : public QLineEdit
     IRCInput(QWidget* parent);
     ~IRCInput();
 
-    void setCompletionMode(char mode) { completionMode=mode; };
-    char getCompletionMode() { return completionMode; };
-    void setOldCursorPosition(int pos) { oldPos=pos; };
-    int getOldCursorPosition() { return oldPos; };
+    void setCompletionMode(char mode);
+    char getCompletionMode();
+    void setOldCursorPosition(int pos);
+    int getOldCursorPosition();
 
   signals:
     void nickCompletion();
     void history(bool up);
     void pageUp();
     void pageDown();
+    void textPasted(QString text);
 
+  public slots:
+    void paste();
+  
   protected slots:
     void getHistory(bool up);
 
