@@ -11,11 +11,13 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: konversationapplication.cpp,v 1.29 2002/12/14 13:12:45 eisfuchs Exp $
+  $Id: konversationapplication.cpp,v 1.43 2003/02/05 21:53:09 eisfuchs Exp $
 */
 
 #include <kdebug.h>
 
+#include <qvaluelist.h>
+
 #include "konversationapplication.h"
 #include "serverwindow.h"
 
@@ -27,27 +29,49 @@ KonversationApplication::KonversationApplication()
 {
   kdDebug() << "KonversationApplication::KonversationApplication()" << endl;
 
+  prefsDialog=0;
+  
+  preferences.setTextFont(font());
+  preferences.setListFont(font());
+
   config=new KSimpleConfig("konversationrc");
   readOptions();
-  prefsDialog=new PrefsDialog(&preferences,true);
 
-  connect(prefsDialog,SIGNAL (connectToServer(int)),this,SLOT (connectToServer(int)) );
-  connect(prefsDialog,SIGNAL (cancelClicked()),this,SLOT (quitKonversation()) );
-  connect(prefsDialog,SIGNAL (prefsChanged()),this,SLOT (saveOptions()) );
+  // handle autoconnect on startup
+  QValueList<int> list=preferences.getAutoConnectServerIDs();
+  // if there is at least one autoconnect server, start connecting right away
+  if(list.count())
+  {
+    for(unsigned int index=0;index<list.count();index++) connectToServer(list[index]);
+    // maybe the user wants to see the prefs dialog anyway
+    if(preferences.getShowServerList())
+    {
+      openPrefsDialog();
+    }
+  }
+  // no autoconnect server, so show the prefs dialog (with exit functionality)
+  else
+  {
+    prefsDialog=new PrefsDialog(&preferences,true);
 
-  /* TODO: Check if this serverList is needed anyway */
-  serverList.setAutoDelete(true);     // delete items when they are removed
+    connect(prefsDialog,SIGNAL (connectToServer(int)),this,SLOT (connectToServer(int)) );
+    connect(prefsDialog,SIGNAL (cancelClicked()),this,SLOT (quitKonversation()) );
+    connect(prefsDialog,SIGNAL (prefsChanged()),this,SLOT (saveOptions()) );
 
-  prefsDialog->show();
+    // TODO: Check if this serverList is needed at all
+    serverList.setAutoDelete(true);     // delete items when they are removed
 
-  connect(&preferences,SIGNAL (requestServerConnection(int)),this,SLOT (connectToAnotherServer(int)) );
-  connect(&preferences,SIGNAL (requestSaveOptions()),this,SLOT (saveOptions()) );
+    prefsDialog->show();
+
+    connect(&preferences,SIGNAL (requestServerConnection(int)),this,SLOT (connectToAnotherServer(int)) );
+    connect(&preferences,SIGNAL (requestSaveOptions()),this,SLOT (saveOptions()) );
+  }
 }
 
 KonversationApplication::~KonversationApplication()
 {
   kdDebug() << "KonversationApplication::~KonversationApplication()" << endl;
-  saveOptions();
+  saveOptions(false);
 
 /*
   kdDebug() << "Deleting Server list ..." << endl;
@@ -64,9 +88,10 @@ void KonversationApplication::connectToServer(int id)
   kdDebug() << "KonversationApplication::connectToServer(" << id << ")" << endl;
 
   connectToAnotherServer(id);
-  /* to prevent doubleClicked() to crash the dialog */
-  /* FIXME: Seems to have a race, though */
-  prefsDialog->delayedDestruct();
+  // to prevent doubleClicked() to crash the dialog
+  // FIXME: Seems to have a race, though
+  // only close the dialog when we didn't use autoconnect
+  if(prefsDialog) prefsDialog->delayedDestruct();
   prefsDialog=0;
 }
 
@@ -148,16 +173,35 @@ void KonversationApplication::readOptions()
   reason=config->readEntry("KickReason","");
   if(reason!="") preferences.setKickReason(reason);
 
+  // Appearance
+  config->setGroup("Appearance");
+  // Fonts
+  preferences.setTextFontRaw(config->readEntry("TextFont",preferences.getTextFont().rawName()));
+  preferences.setListFontRaw(config->readEntry("ListFont",preferences.getListFont().rawName()));
+  preferences.setTimestamping(config->readBoolEntry("Timestamping",preferences.getTimestamping()));
+  preferences.setTimestampFormat(config->readEntry("TimestampFormat",preferences.getTimestampFormat()));
+  preferences.setShowQuickButtons(config->readBoolEntry("ShowQuickButtons",preferences.getShowQuickButtons()));
+  preferences.setShowModeButtons(config->readBoolEntry("ShowModeButtons",preferences.getShowModeButtons()));
+  preferences.setCodec(config->readEntry("Codec",preferences.getCodec()));
+
+  QValueList<int> sizes;
+  QString sizesString=config->readEntry("ChannelSplitter","10,1");
+  sizes.append(sizesString.section(',',0,0).toInt());
+  sizes.append(sizesString.section(',',1,1).toInt());
+  preferences.setChannelSplitter(sizes);
+  
   // Colors
   config->setGroup("Message Text Colors");
 
-  preferences.setActionMessageColor(config->readEntry("ActionMessage", preferences.defaultActionMessageColor));
-  preferences.setBacklogMessageColor(config->readEntry("BacklogMessage", preferences.defaultBacklogMessageColor));
-  preferences.setChannelMessageColor(config->readEntry("ChannelMessage", preferences.defaultChannelMessageColor));
-  preferences.setCommandMessageColor(config->readEntry("CommandMessage", preferences.defaultCommandMessageColor));
-  preferences.setLinkMessageColor(config->readEntry("LinkMessage", preferences.defaultLinkMessageColor));
-  preferences.setQueryMessageColor(config->readEntry("QueryMessage", preferences.defaultQueryMessageColor));
-  preferences.setServerMessageColor(config->readEntry("ServerMessage", preferences.defaultServerMessageColor));
+  preferences.setActionMessageColor(config->readEntry("ActionMessage", preferences.getActionMessageColor()));
+  preferences.setBacklogMessageColor(config->readEntry("BacklogMessage", preferences.getBacklogMessageColor()));
+  preferences.setChannelMessageColor(config->readEntry("ChannelMessage", preferences.getChannelMessageColor()));
+  preferences.setCommandMessageColor(config->readEntry("CommandMessage", preferences.getCommandMessageColor()));
+  preferences.setLinkMessageColor(config->readEntry("LinkMessage", preferences.getLinkMessageColor()));
+  preferences.setQueryMessageColor(config->readEntry("QueryMessage", preferences.getQueryMessageColor()));
+  preferences.setServerMessageColor(config->readEntry("ServerMessage", preferences.getServerMessageColor()));
+  preferences.setTimeColor(config->readEntry("Time", preferences.getTimeColor()));
+  preferences.setTextViewBackground(config->readEntry("TextViewBackground", preferences.getTextViewBackground()));
 
   // Led Colors
   config->setGroup("Led Colors");
@@ -173,6 +217,10 @@ void KonversationApplication::readOptions()
   QString nickList=config->readEntry("Nicknames",preferences.getNicknameList().join(","));
   preferences.setNicknameList(QStringList::split(",",nickList));
 
+  preferences.setShowAwayMessage(config->readBoolEntry("ShowAwayMessage",preferences.getShowAwayMessage()));
+  preferences.setAwayMessage(config->readEntry("AwayMessage",preferences.getAwayMessage()));
+  preferences.setUnAwayMessage(config->readEntry("UnAwayMessage",preferences.getUnAwayMessage()));
+
   // Notify Settings and list
   config->setGroup("Notify List");
   preferences.setNotifyDelay(config->readNumEntry("NotifyDelay",20));
@@ -222,6 +270,14 @@ void KonversationApplication::readOptions()
   else
     preferences.setHilightNickColor("#"+hilight);
 
+  preferences.setHilightOwnLines(config->readBoolEntry("HilightOwnLines",preferences.getHilightOwnLines()));
+  hilight=config->readEntry("HilightOwnLinesColor");
+  if(hilight=="")
+    preferences.setHilightOwnLinesColor(preferences.getHilightOwnLinesColor().name());
+  else
+    preferences.setHilightOwnLinesColor("#"+hilight);
+
+    
   // Ignore List
   config->setGroup("Ignore List");
   // Remove all default entries if there is at least one Ignore in the preferences file
@@ -265,9 +321,10 @@ void KonversationApplication::readOptions()
   preferences.setAutoRejoin(config->readBoolEntry("AutoRejoin",preferences.getAutoRejoin()));
 
   preferences.setFixedMOTD(config->readBoolEntry("FixedMOTD",preferences.getFixedMOTD()));
+  preferences.setShowServerList(config->readBoolEntry("ShowServerList",preferences.getShowServerList()));
 }
 
-void KonversationApplication::saveOptions()
+void KonversationApplication::saveOptions(bool updateGUI)
 {
   kdDebug() << "KonversationApplication::saveOptions()" << endl;
 
@@ -294,6 +351,19 @@ void KonversationApplication::saveOptions()
   config->writeEntry("PartReason",preferences.getPartReason());
   config->writeEntry("KickReason",preferences.getKickReason());
 
+  config->setGroup("Appearance");
+
+  config->writeEntry("TextFont",preferences.getTextFont().toString());
+  config->writeEntry("ListFont",preferences.getListFont().toString());
+  config->writeEntry("Timestamping",preferences.getTimestamping());
+  config->writeEntry("TimestampFormat",preferences.getTimestampFormat());
+  config->writeEntry("ShowQuickButtons",preferences.getShowQuickButtons());
+  config->writeEntry("ShowModeButtons",preferences.getShowModeButtons());
+  config->writeEntry("Codec",preferences.getCodec());
+
+  QString sizesString(QString::number(preferences.getChannelSplitter()[0])+","+QString::number(preferences.getChannelSplitter()[1]));
+  config->writeEntry("ChannelSplitter",sizesString);
+
   config->setGroup("Message Text Colors");
 
   config->writeEntry("ActionMessage", preferences.getActionMessageColor());
@@ -303,6 +373,8 @@ void KonversationApplication::saveOptions()
   config->writeEntry("LinkMessage", preferences.getLinkMessageColor());
   config->writeEntry("QueryMessage", preferences.getQueryMessageColor());
   config->writeEntry("ServerMessage", preferences.getServerMessageColor());
+  config->writeEntry("Time", preferences.getTimeColor());
+  config->writeEntry("TextViewBackground", preferences.getTextViewBackground());
 
   config->setGroup("Led Colors");
   config->writeEntry("OperatorColor", preferences.getOpLedColor());
@@ -314,8 +386,12 @@ void KonversationApplication::saveOptions()
   config->writeEntry("Ident",preferences.ident);
   config->writeEntry("Realname",preferences.realname);
   config->writeEntry("Nicknames",preferences.getNicknameList());
+  config->writeEntry("ShowAwayMessage",preferences.getShowAwayMessage());
+  config->writeEntry("AwayMessage",preferences.getAwayMessage());
+  config->writeEntry("UnAwayMessage",preferences.getUnAwayMessage());
 
   config->setGroup("Notify List");
+
   config->writeEntry("NotifyDelay",preferences.getNotifyDelay());
   config->writeEntry("UseNotify",preferences.getUseNotify());
   config->writeEntry("NotifyList",preferences.getNotifyString());
@@ -354,10 +430,14 @@ void KonversationApplication::saveOptions()
   hilight=hilight.stripWhiteSpace();
   // write hilight string
   config->writeEntry("Hilight",hilight);
+
   config->writeEntry("HilightNick",preferences.getHilightNick());
   config->writeEntry("HilightNickColor",preferences.getHilightNickColor().name().mid(1));
+  config->writeEntry("HilightOwnLines",preferences.getHilightOwnLines());
+  config->writeEntry("HilightOwnLinesColor",preferences.getHilightOwnLinesColor().name().mid(1));
 
   // Ignore List
+  config->deleteGroup("Ignore List");
   config->setGroup("Ignore List");
   QPtrList<Ignore> ignoreList=preferences.getIgnoreList();
   Ignore* item=ignoreList.first();
@@ -401,8 +481,23 @@ void KonversationApplication::saveOptions()
   config->writeEntry("AutoRejoin",preferences.getAutoRejoin());
 
   config->writeEntry("FixedMOTD",preferences.getFixedMOTD());
+  config->writeEntry("ShowServerList",preferences.getShowServerList());
 
   config->sync();
+
+  if(updateGUI)
+  {
+    Server* lookServer=serverList.first();
+    while(lookServer)
+    {
+      // TODO: This also updates the background color! We must finally
+      // find a way to do all this with signals / slots!
+      lookServer->updateFonts();
+      lookServer->setShowQuickButtons(preferences.getShowQuickButtons());
+      lookServer->setShowModeButtons(preferences.getShowModeButtons());
+      lookServer=serverList.next();
+    }
+  }
 }
 
 
@@ -435,3 +530,6 @@ void KonversationApplication::closePrefsDialog()
   delete prefsDialog;
   prefsDialog=0;
 }
+
+#include "konversationapplication.moc"
+
