@@ -11,15 +11,15 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: konversationapplication.h,v 1.5 2002/07/24 14:33:41 eisfuchs Exp $
+  $Id: konversationapplication.h,v 1.7 2003/02/05 16:43:54 eisfuchs Exp $
 */
 
-#include <kapp.h>
-#include <ksimpleconfig.h>
-
 #ifndef KONVERSATIONAPPLICATION_H
 #define KONVERSATIONAPPLICATION_H
 
+#include <kapp.h>
+#include <ksimpleconfig.h>
+
 #include "preferences.h"
 #include "prefsdialog.h"
 #include "server.h"
@@ -47,7 +47,7 @@ class KonversationApplication : public KApplication
     void connectToServer(int number);
     void connectToAnotherServer(int number);
     void readOptions();
-    void saveOptions();
+    void saveOptions(bool updateGUI=true);
     void quitKonversation();
 
     void openPrefsDialog();
