@@ -11,7 +11,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: ledtabbar.h,v 1.4 2002/07/27 21:46:07 eisfuchs Exp $
+  $Id: ledtabbar.h,v 1.5 2003/01/29 06:34:30 eisfuchs Exp $
 */
 
 #ifndef LEDTABBAR_H
@@ -35,6 +35,9 @@ class LedTabBar : public QTabBar
 
     LedTab* tab(int id);
     LedTab* tab(QWidget* widget);
+
+  public slots:
+    void repaintLED(LedTab* tab);
 };
 
 #endif
