@@ -11,7 +11,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: prefspagedccsettings.h,v 1.4 2002/12/01 13:08:53 eisfuchs Exp $
+  $Id: prefspagedccsettings.h,v 1.5 2003/02/06 15:12:02 eisfuchs Exp $
 */
 
 
@@ -34,11 +34,15 @@ class PrefsPageDccSettings : public PrefsPage
 
   protected slots:
     void folderInputChanged(const QString& newPath);
+    void folderButtonClicked();
     void bufferValueChanged(int newBuffer);
     void rollbackValueChanged(int newRollback);
     void autoGetChanged(int state);
     void addSenderChanged(int state);
     void createFolderChanged(int state);
+
+  protected:
+    KLineEdit* dccFolderInput;
 };
 
 #endif
