@@ -11,13 +11,14 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: server.cpp,v 1.31 2002/12/14 13:12:45 eisfuchs Exp $
+  $Id: server.cpp,v 1.46 2003/02/07 10:19:42 eisfuchs Exp $
 */
 
 #include <unistd.h>
 
 #include <qregexp.h>
 #include <qhostaddress.h>
+#include <qtextcodec.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -57,7 +58,7 @@ Server::Server(int id)
   }
   else autoJoin=false;
 
-  autoReconnect=KonversationApplication::preferences.getAutoRejoin();
+  autoRejoin=KonversationApplication::preferences.getAutoRejoin();
   autoReconnect=KonversationApplication::preferences.getAutoReconnect();
 
   serverSocket=0;
@@ -166,6 +167,7 @@ void Server::connectToIRCServer()
       connect(serverSocket,SIGNAL (readyRead()),this,SLOT (incoming()) );
       connect(serverSocket,SIGNAL (readyWrite()),this,SLOT (send()) );
       connect(serverSocket,SIGNAL (closed(int)),this,SLOT (broken(int)) );
+//      connect(serverSocket,SIGNAL (connectionFailed(int)),this,SLOT (broken(int)) );
 
       connect(this,SIGNAL (nicknameChanged(const QString&)),serverWindow,SLOT (setNickname(const QString&)) );
 
@@ -243,6 +245,7 @@ void Server::startNotifyTimer(int msec)
   if(msec==0) msec=KonversationApplication::preferences.getNotifyDelay()*1000; // msec!
   // start the timer in one shot mode
   notifyTimer.start(msec,true);
+  notifyCheckTimer.stop();
   // reset check time
   checkTime=0;
 }
@@ -279,12 +282,17 @@ void Server::notifyTimeout()
 void Server::notifyCheckTimeout()
 {
   checkTime+=500;
-  emit tooLongLag(checkTime);
+  if(serverSocket) emit tooLongLag(checkTime);
 }
 
 QString Server::getAutoJoinCommand()
 {
-  QString autoString("JOIN "+autoJoinChannel+" "+autoJoinChannelKey);
+  // Multichannel joins
+  QStringList channels=QStringList::split(' ',autoJoinChannel);
+  QStringList keys=QStringList::split(' ',autoJoinChannelKey);
+  
+  QString autoString("JOIN "+channels.join(",")+" "+keys.join(","));
+
   return autoString;
 }
 
@@ -322,13 +330,13 @@ void Server::incoming()
   char buffer[513];
   int len=0;
 
-  do
-  {
-    len=read(serverSocket->fd(),buffer,512);
-    buffer[len]=0;
+  len=read(serverSocket->fd(),buffer,512);
 
-    inputBuffer+=buffer;
-  } while(len==512);
+  buffer[len]=0;
+
+  inputBuffer+=buffer;
+
+  if(len==0) broken(0);
 }
 
 void Server::queue(const QString& buffer)
@@ -357,7 +365,8 @@ void Server::send()
     // Don't reconnect if we WANT to quit
     else if(outputBuffer.startsWith("QUIT")) setDeliberateQuit(true);
     // TODO: Implement Flood-Protection here
-    write(serverSocket->fd(),outputBuffer.latin1(),outputBuffer.length());
+//    write(serverSocket->fd(),outputBuffer.latin1(),outputBuffer.length());
+    write(serverSocket->fd(),outputBuffer,outputBuffer.length());
     serverSocket->enableWrite(false);
   }
 
@@ -381,13 +390,15 @@ void Server::setDeliberateQuit(bool on)
 
 void Server::broken(int state)
 {
-  kdWarning() << "Connection broken (Socket fd " << serverSocket->fd() << ") " << state << "!" << endl;
+  kdDebug() << "Connection broken (Socket fd " << serverSocket->fd() << ") " << state << "!" << endl;
 
   serverWindow->appendToStatus(i18n("Error"),i18n("Connection to Server %1 lost.").arg(serverName));
   // TODO: Close all queries and channels!
   //       Or at least make sure that all gets reconnected properly
+  disconnect(serverSocket,0,0,0);
   delete serverSocket;
   serverSocket=0;
+  kdDebug() << "Socket deleted" << endl;
 
   if(autoReconnect && !getDeliberateQuit())
   {
@@ -714,6 +725,51 @@ void Server::updateChannelQuickButtons(QStringList newButtons)
   }
 }
 
+// TODO: Maybe use a Signal / Slot mechanism for these things?
+void Server::updateFonts()
+{
+  kdDebug() << "Server::updateFonts()" << endl;
+
+  Channel* channel=channelList.first();
+  while(channel)
+  {
+    channel->updateFonts();
+    channel=channelList.next();
+  }
+
+  Query* query=queryList.first();
+  while(query)
+  {
+    query->updateFonts();
+    query=queryList.next();
+  }
+
+  // TODO: To be revised
+  if(serverWindow) serverWindow->updateFonts();
+}
+
+// TODO: Maybe use a Signal / Slot mechanism for these things?
+void Server::setShowQuickButtons(bool state)
+{
+  Channel* channel=channelList.first();
+  while(channel)
+  {
+    channel->showQuickButtons(state);
+    channel=channelList.next();
+  }
+}
+
+// TODO: Maybe use a Signal / Slot mechanism for these things?
+void Server::setShowModeButtons(bool state)
+{
+  Channel* channel=channelList.first();
+  while(channel)
+  {
+    channel->showModeButtons(state);
+    channel=channelList.next();
+  }
+}
+
 Channel* Server::getChannelByName(const char* name)
 {
   // Convert wanted channel name to lowercase
@@ -778,13 +834,19 @@ void Server::addHostmaskToNick(QString& sourceNick,QString& sourceHostmask)
 void Server::removeNickFromChannel(QString& channelName,QString& nickname,QString& reason,bool quit)
 {
   Channel* outChannel=getChannelByName(channelName);
-  if(outChannel) outChannel->removeNick(nickname,reason,quit);
+  if(outChannel)
+  {
+    if(outChannel->getNickByName(nickname)) outChannel->removeNick(nickname,reason,quit);
+  }
 }
 
 void Server::nickWasKickedFromChannel(QString& channelName,QString& nickname,QString& kicker,QString& reason)
 {
   Channel* outChannel=getChannelByName(channelName);
-  if(outChannel) outChannel->kickNick(nickname,kicker,reason);
+  if(outChannel)
+  {
+    if(outChannel->getNickByName(nickname)) outChannel->kickNick(nickname,kicker,reason);
+  }
 }
 
 void Server::removeNickFromServer(QString& nickname,QString& reason)
@@ -803,7 +865,7 @@ void Server::renameNick(QString& nickname,QString& newNick)
   Channel* channel=channelList.first();
   while(channel)
   {
-    channel->renameNick(nickname,newNick);
+    if(channel->getNickByName(nickname)) channel->renameNick(nickname,newNick);
     channel=channelList.next();
   }
   // If this was our own nickchange, tell our server object about it
@@ -884,16 +946,26 @@ void Server::setNickname(const QString& newNickname)
   serverWindow->setNickname(newNickname);
 }
 
-void Server::setChannelTopic(QString& channel,QString &topic)
+void Server::setChannelTopic(QString& channel,QString& newTopic)
 {
   Channel* outChannel=getChannelByName(channel);
-  if(outChannel) outChannel->setTopic(topic);
+  if(outChannel)
+  {
+    QTextCodec* codec=QTextCodec::codecForName(KonversationApplication::preferences.getCodec());
+    QString topic=codec->toUnicode(newTopic);
+    outChannel->setTopic(topic);
+  }
 }
 
-void Server::setChannelTopic(QString& nickname,QString& channel,QString &topic) // Overloaded
+void Server::setChannelTopic(QString& nickname,QString& channel,QString& newTopic) // Overloaded
 {
   Channel* outChannel=getChannelByName(channel);
-  if(outChannel) outChannel->setTopic(nickname,topic);
+  if(outChannel)
+  {
+    QTextCodec* codec=QTextCodec::codecForName(KonversationApplication::preferences.getCodec());
+    QString topic=codec->toUnicode(newTopic);
+    outChannel->setTopic(nickname,newTopic);
+  }
 }
 
 bool Server::isNickname(QString& compare)
@@ -967,3 +1039,27 @@ void Server::setServerWindow(ServerWindow* newWindow)
 {
   serverWindow=newWindow;
 }
+
+void Server::away()
+{
+  isAway=true;
+}
+
+void Server::sendToAllChannels(const QString& text)
+{
+  // Send a message to all channels we are in
+  Channel* channel=channelList.first();
+
+  while(channel)
+  {
+    channel->sendChannelText(text);
+    channel=channelList.next();
+  }
+}
+
+void Server::unAway()
+{
+  isAway=false;
+}
+
+#include "server.moc"
