@@ -11,7 +11,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: serverlistitem.cpp,v 1.4 2002/07/20 15:18:19 eisfuchs Exp $
+  $Id: serverlistitem.cpp,v 1.5 2003/01/25 19:22:58 eisfuchs Exp $
 */
 
 #include <kdebug.h>
@@ -25,12 +25,25 @@ ServerListItem::ServerListItem(QListView* parent,int newId,
                                QString arg3,
                                QString arg4,
                                QString arg5) :
-                 KListViewItem(parent,arg0,arg1,arg2,arg3,arg4,arg5)
+                QCheckListItem(parent,QString::null,QCheckListItem::CheckBox)
 {
   id=newId;
+  setText(1,arg0);
+  setText(2,arg1);
+  setText(3,arg2);
+  setText(4,arg3);
+  setText(5,arg4);
+  setText(6,arg5);
 }
 
 ServerListItem::~ServerListItem()
 {
   kdDebug() << "ServerListItem::~ServerListItem(" << text(0) << ")" << endl;
 }
+
+void ServerListItem::stateChange(bool state)
+{
+  emit autoStateChanged(this,state);
+}
+
+#include "serverlistitem.moc"
