@@ -11,7 +11,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: serverwindow.cpp,v 1.28 2002/12/14 13:12:45 eisfuchs Exp $
+  $Id: serverwindow.cpp,v 1.38 2003/02/05 20:16:57 eisfuchs Exp $
 */
 
 #include <qdir.h>
@@ -19,6 +19,7 @@
 #include <klocale.h>
 #include <kdebug.h>
 #include <kdialog.h>
+#include <kaccel.h>
 
 #include "serverwindow.h"
 #include "konversationapplication.h"
@@ -46,18 +47,23 @@ ServerWindow::ServerWindow(Server* newServer) : KMainWindow()
   showStatusBarAction=KStdAction::showStatusbar(this,SLOT(showStatusbar()),actionCollection()); /* options_show_statusbar */
 /*  KAction* prefsAction= */ KStdAction::preferences(this,SLOT(openPreferences()),actionCollection()); /* options_configure */
 /*  KAction* open_quickbuttons_action= */ new KAction(i18n("Buttons"),0,0,this,SLOT (openButtons()),actionCollection(),"open_buttons_window");
-/*  KAction* open_hilight_action=      */ new KAction(i18n("Hilight List"),0,0,this,SLOT (openHilight()),actionCollection(),"open_hilight_window");
+/*  KAction* open_hilight_action=      */ new KAction(i18n("Highlight List"),0,0,this,SLOT (openHilight()),actionCollection(),"open_hilight_window");
 /*  KAction* open_notify_action=       */ new KAction(i18n("Notify List"),0,0,this,SLOT (openNotify()),actionCollection(),"open_notify_window");
 /*  KAction* open_nicksonline_action=  */ new KAction(i18n("Nicks Online"), 0, 0, this, SLOT(openNicksOnlineWindow()), actionCollection(), "open_nicksonline_window");
 /*  KAction* open_ignore_action=       */ new KAction(i18n("Ignore List"),0,0,this,SLOT (openIgnore()),actionCollection(),"open_ignore_window");
 /*  KAction* open_colors_action=       */ new KAction(i18n("Configure Colors"), 0, 0, this, SLOT(openColorConfiguration()), actionCollection(), "open_colors_window");
   setCentralWidget(windowContainer);
 
-  /* Initialize KMainWindow->statusBar() */
+  // Keyboard accelerators to navigate through the different pages
+  KAccel* accelerator=accel();
+  accelerator->insert("Next Tab",i18n("Next Tab"),"Jump to next tab",KShortcut("Alt+Right"),this,SLOT(nextTab()));
+  accelerator->insert("Previous Tab",i18n("Previous Tab"),"Jump to previous tab",KShortcut("Alt+Left"),this,SLOT(previousTab()));
+
+  // Initialize KMainWindow->statusBar()
   statusBar();
   statusBar()->insertItem(i18n("Ready."),StatusText,1);
   statusBar()->insertItem("lagometer",LagOMeter,0,true);
-  /* Show "Lag unknown" */
+  // Show "Lag unknown"
   resetLag();
   statusBar()->setItemAlignment(StatusText,QLabel::AlignLeft);
 
@@ -106,9 +112,8 @@ void ServerWindow::setServer(Server* newServer)
     delete nicksOnlineWindow;
     nicksOnlineWindow=0;
   }
+
   server=newServer;
-  connect(&filter,SIGNAL (openQuery(const QString&,const QString&)),
-           server,SLOT   (addQuery(const QString&,const QString&)) );
 }
 
 Server* ServerWindow::getServer()
@@ -118,59 +123,70 @@ Server* ServerWindow::getServer()
 
 void ServerWindow::appendToStatus(const QString& type,const QString& message)
 {
-  statusView->appendServerMessage(type,message);
-  /* Show activity indicator */
-  newText(statusPane);
+  statusPanel->appendServerMessage(type,message);
 }
 
 void ServerWindow::appendToFrontmost(const QString& type,const QString& message)
 {
-  /* FIXME: Make Status Pane a Chat Window to get rid of exceptions */
-  if(frontView==0)
+  // TODO: Make it an option to direct all status stuff into the status panel
+
+  if(frontView==0) // Check if the frontView can actually display text
   {
-    statusView->appendServerMessage(type,message);
-    /* Show activity indicator */
-    newText(statusPane);
+    statusPanel->appendServerMessage(type,message);
+    newText(statusPanel);
   }
   else
     frontView->appendServerMessage(type,message);
 }
 
-void ServerWindow::statusTextEntered()
-{
-  QString line=statusInput->text();
-  if(line.lower()=="/clear") statusView->clear();  // FIXME: to get rid of too wide lines
-  else
-  {
-    QString output=filter.parse(server->getNickname(),line,"");
-
-    if(output!="") appendToStatus(filter.getType(),output);
-
-    server->queue(filter.getServerOutput());
-  }
-
-  statusInput->clear();
-}
-
 void ServerWindow::addView(QWidget* pane,int color,const QString& label)
 {
-  /* TODO: Make sure to add DCC status tab at the end of the list and all others */
-  /* before the DCC tab. Maybe we should also make sure to order Channels */
-  /* Queries and DCC chats in groups */
+  // TODO: Make sure to add DCC status tab at the end of the list and all others
+  // before the DCC tab. Maybe we should also make sure to order Channels
+  // Queries and DCC chats in groups
   windowContainer->addTab(pane,label,color,true);
   // TODO: Check, if user was typing in old input line
   if(KonversationApplication::preferences.getBringToFront())
   {
-    // Don't bring Tab to front if TabWidget is hidden. Otherwise QT gets confused
-    // and shows the Tab as active but will display the wrong pane
-    if(windowContainer->isVisible())
-      showView(pane);
+    showView(pane);
   }
 }
 
 void ServerWindow::showView(QWidget* pane)
 {
-  windowContainer->showPage(pane);
+  // Don't bring Tab to front if TabWidget is hidden. Otherwise QT gets confused
+  // and shows the Tab as active but will display the wrong pane
+  if(windowContainer->isVisible())
+  {
+    // TODO: add adjustFocus() here?
+    windowContainer->showPage(pane);
+  }
+}
+
+void ServerWindow::nextTab()
+{
+  kdDebug() << "ServerWindow::nextTab()" << endl;
+
+  int page=windowContainer->currentPageIndex()+1;
+  if(page<windowContainer->count())
+  {
+    windowContainer->setCurrentPage(page);
+    ChatWindow* newPage=(ChatWindow*) windowContainer->page(page);
+    newPage->adjustFocus();
+  }
+}
+
+void ServerWindow::previousTab()
+{
+  kdDebug() << "ServerWindow::previousTab()" << endl;
+
+  int page=windowContainer->currentPageIndex()-1;
+  if(page!=-1)
+  {
+    windowContainer->setCurrentPage(page);
+    ChatWindow* newPage=(ChatWindow*) windowContainer->page(page);
+    newPage->adjustFocus();
+  }
 }
 
 void ServerWindow::addDccPanel()
@@ -223,70 +239,23 @@ void ServerWindow::deleteDccPanel()
 
 void ServerWindow::addStatusView()
 {
-  statusPane=new QVBox(windowContainer);
-  statusPane->setSpacing(spacing());
-  statusPane->setMargin(margin());
+  statusPanel=new StatusPanel(getWindowContainer());
+  addView(statusPanel,2,i18n("Status"));
+  statusPanel->setServer(getServer());
 
-  statusView=new IRCView(statusPane,getServer());
-
-  QHBox* commandLineBox=new QHBox(statusPane);
-  commandLineBox->setSpacing(spacing());
-  commandLineBox->setMargin(0);
-
-  nicknameButton=new QPushButton(i18n("Nickname"),commandLineBox);
-  statusInput=new IRCInput(commandLineBox);
-  logCheckBox=new QCheckBox(i18n("Log"),commandLineBox);
-  logCheckBox->setChecked(KonversationApplication::preferences.getLog());
-
-  windowContainer->addTab(statusPane,i18n("Status"),2,false);
-
-  connect(statusInput,SIGNAL (returnPressed()),this,SLOT(statusTextEntered()) );
-  connect(statusView,SIGNAL (gotFocus()),statusInput,SLOT (setFocus()) );
-  connect(statusView,SIGNAL (textToLog(const QString&)),this,SLOT (logText(const QString&)) );
-
-  setLog(KonversationApplication::preferences.getLog());
-}
-
-void ServerWindow::logText(const QString& text)
-{
-  QDir logPath=QDir::home();
-
-  /* Try to "cd" into the logfile path */
-  if(!logPath.cd(KonversationApplication::preferences.getLogPath(),true))
-  {
-    /* Try to create the logfile path and "cd" into it again */
-    logPath.mkdir(KonversationApplication::preferences.getLogPath(),true);
-    logPath.cd(KonversationApplication::preferences.getLogPath(),true);
-  }
-
-  /* add the logfile name to the path */
-  logfile.setName(logPath.path()+"/konversation.log");
-
-  if(logfile.open(IO_WriteOnly | IO_Append))
-  {
-    if(firstLog)
-    {
-      QString intro(i18n("\n*** Logfile started\n*** on %1\n\n").arg(QDateTime::currentDateTime().toString()));
-      logfile.writeBlock(intro,intro.length());
-      firstLog=false;
-    }
-
-    QTime time=QTime::currentTime();
-    QString logLine(QString("[%1] %2\n").arg(time.toString("hh:mm:ss")).arg(text));
-    logfile.writeBlock(logLine,logLine.length());
-    logfile.close();
-  }
-  else kdWarning() << "ServerWindow::logText(): open(IO_Append) for " << logfile.name() << " failed!" << endl;
+  connect(statusPanel,SIGNAL (newText(QWidget*)),this,SLOT (newText(QWidget*)) );
 }
 
 void ServerWindow::setNickname(const QString& newNickname)
 {
-  nicknameButton->setText(newNickname);
+  // TODO: connect this to the appropriate server's nickname signal
+  statusPanel->setNickname(newNickname);
 }
 
 void ServerWindow::newText(QWidget* view)
 {
-  /* FIXME: Should be compared to ChatWindow* but the status Window currently is something else */
+  // FIXME: Should be compared to ChatWindow* but the status Window currently is something else
+  // Now that the status Window is a ChatWindow* we can start cleaning up here
   if(view!=(QWidget*) windowContainer->currentPage())
   {
     windowContainer->changeTabState(view,true);
@@ -296,26 +265,24 @@ void ServerWindow::newText(QWidget* view)
 void ServerWindow::changedView(QWidget* view)
 {
   frontView=0;
-  if(view!=statusPane)
-  {
-    /* FIXME: Make status window a Chat Window to get rid of those exceptions */
-    ChatWindow* pane=(ChatWindow*) view;
-    if(pane->getType()==ChatWindow::Channel || pane->getType()==ChatWindow::Query) frontView=pane;
-  }
+  
+  ChatWindow* pane=(ChatWindow*) view;
+  // Make sure that only text-capable views get to be the frontView
+  if(pane->getType()!=ChatWindow::DccPanel) frontView=pane;
 
   windowContainer->changeTabState(view,false);
 }
 
 void ServerWindow::readOptions()
 {
-  /* Tool bar settings */
+  // Tool bar settings
   showToolBarAction->setChecked(KonversationApplication::preferences.serverWindowToolBarStatus);
   toolBar("mainToolBar")->setBarPos((KToolBar::BarPosition) KonversationApplication::preferences.serverWindowToolBarPos);
   toolBar("mainToolBar")->setIconText((KToolBar::IconText) KonversationApplication::preferences.serverWindowToolBarIconText);
   toolBar("mainToolBar")->setIconSize(KonversationApplication::preferences.serverWindowToolBarIconSize);
   showToolbar();
 
-  /* Status bar settings */
+  // Status bar settings
   showStatusBarAction->setChecked(KonversationApplication::preferences.serverWindowStatusBarStatus);
   showStatusbar();
 
@@ -325,7 +292,8 @@ void ServerWindow::readOptions()
     resize(size);
   }
 }
-/* Will not actually save the options but write them into the prefs structure */
+
+// Will not actually save the options but write them into the prefs structure
 void ServerWindow::saveOptions()
 {
   KonversationApplication::preferences.setServerWindowSize(size());
@@ -455,7 +423,7 @@ void ServerWindow::applyNotify(QStringList newList,bool use,int delay)
   KonversationApplication::preferences.setNotifyDelay(delay);
   KonversationApplication::preferences.setUseNotify(use);
 
-  /* Restart notify timer if desired */
+  // Restart notify timer if desired
   if(use) server->startNotifyTimer();
 
   emit prefsChanged();
@@ -500,9 +468,12 @@ void ServerWindow::openColorConfiguration()
 																										KonversationApplication::preferences.getLinkMessageColor(),
 																										KonversationApplication::preferences.getQueryMessageColor(),
 																										KonversationApplication::preferences.getServerMessageColor(),
+																										KonversationApplication::preferences.getTimeColor(),
+																										KonversationApplication::preferences.getTextViewBackground(),
 																										KonversationApplication::preferences.getColorConfigurationSize());
-	connect(colorConfigurationDialog, SIGNAL(saveFontColorSettings(QString, QString, QString, QString, QString, QString, QString)),
-					this, SLOT(applyColorConfiguration(QString, QString, QString, QString, QString, QString, QString)));
+
+  connect(colorConfigurationDialog, SIGNAL(saveFontColorSettings(QString, QString, QString, QString, QString, QString, QString, QString, QString)),
+					this, SLOT(applyColorConfiguration(QString, QString, QString, QString, QString, QString, QString, QString, QString)));
 	connect(colorConfigurationDialog, SIGNAL(closeFontColorConfiguration(QSize)),
 					this, SLOT(closeColorConfiguration(QSize)));
 
@@ -511,7 +482,7 @@ void ServerWindow::openColorConfiguration()
 
 void ServerWindow::applyColorConfiguration(QString actionTextColor, QString backlogTextColor, QString channelTextColor,
 														 							 QString commandTextColor, QString linkTextColor, QString queryTextColor,
-																					 QString serverTextColor)
+																					 QString serverTextColor, QString timeColor, QString backgroundColor)
 {
 	KonversationApplication::preferences.setActionMessageColor(actionTextColor);
 	KonversationApplication::preferences.setBacklogMessageColor(backlogTextColor);
@@ -520,6 +491,8 @@ void ServerWindow::applyColorConfiguration(QString actionTextColor, QString back
 	KonversationApplication::preferences.setLinkMessageColor(linkTextColor);
 	KonversationApplication::preferences.setQueryMessageColor(queryTextColor);
 	KonversationApplication::preferences.setServerMessageColor(serverTextColor);
+	KonversationApplication::preferences.setTimeColor(timeColor);
+	KonversationApplication::preferences.setTextViewBackground(backgroundColor);
 	emit prefsChanged();
 }
 
@@ -527,8 +500,8 @@ void ServerWindow::closeColorConfiguration(QSize windowSize)
 {
 	KonversationApplication::preferences.setColorConfigurationSize(windowSize);
 	emit prefsChanged();
-	disconnect(colorConfigurationDialog, SIGNAL(saveFontColorSettings(QString, QString, QString, QString, QString, QString, QString)),
-				 		 this, SLOT(applyColorConfiguration(QString, QString, QString, QString, QString, QString, QString)));
+	disconnect(colorConfigurationDialog, SIGNAL(saveFontColorSettings(QString, QString, QString, QString, QString, QString, QString, QString, QString)),
+				 		 this, SLOT(applyColorConfiguration(QString, QString, QString, QString, QString, QString, QString, QString, QString)));
 	disconnect(colorConfigurationDialog, SIGNAL(closeFontColorConfiguration(QSize)),
 						 this, SLOT(closeColorConfiguration(QSize)));
 	delete colorConfigurationDialog;
@@ -540,11 +513,6 @@ void ServerWindow::channelPrefsChanged()
   emit prefsChanged();
 }
 
-void ServerWindow::setLog(bool activated)
-{
-  log=activated;
-}
-
 LedTabWidget* ServerWindow::getWindowContainer()
 {
   return windowContainer;
@@ -560,6 +528,13 @@ int ServerWindow::margin()
   return KDialog::marginHint();
 }
 
+void ServerWindow::updateFonts()
+{
+  kdDebug() << "ServerWindow::updateFonts()" << endl;
+
+  statusPanel->updateFonts();  // FIXME: should be done by the respective server, no?
+}
+
 void ServerWindow::updateLag(int msec)
 {
   statusBar()->changeItem(i18n("Ready."),StatusText);
@@ -584,3 +559,10 @@ void ServerWindow::resetLag()
 {
   statusBar()->changeItem(i18n("Lag: not known"),LagOMeter);
 }
+
+bool ServerWindow::isFrontView(const ChatWindow* view)
+{
+  return(frontView==view);
+}
+
+#include "serverwindow.moc"
