@@ -11,12 +11,15 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 
-  $Id: topiccombobox.cpp,v 1.1 2002/11/21 20:44:20 eisfuchs Exp $
+  $Id: topiccombobox.cpp,v 1.5 2003/02/07 10:19:42 eisfuchs Exp $
 */
 
 #include <kdebug.h>
 
+#include <qtextcodec.h>
+
 #include "topiccombobox.h"
+#include "konversationapplication.h"
 
 TopicComboBox::TopicComboBox(QWidget* parent) :
                    KComboBox(parent)
@@ -46,5 +49,8 @@ void TopicComboBox::topicActivated(int index)
 void TopicComboBox::insertStringList(const QStringList& list)
 {
   KComboBox::insertStringList(list);
+
   setEditText(list[0].section(' ',1));
 }
+
+#include "topiccombobox.moc"
